/***

IPTraf
An IP-oriented Network Statistics Utility for
Ethernet and SLIP/PPP networks
   		
Written by Gerard Paul Java
Copyright (c) Gerard Paul Java 1997, 1998
   	
Version 1.0.3

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***/

#include <ncurses.h>
#include <panel.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include "deskman.h"
#include "menurt.h"
#include "stdwinset.h"
#include "utfdefs.h"
#include "othfilter.h"
#include "utfilter.h"
#include "ifstats.h"
#include "serv.h"
#include "options.h"
#include "externs.h"
#include "log.h"
#include "attrs.h"

extern void srpromisc(int mode);
extern void about();
/*
 * Get the ball rolling: The program interface routine.
 */

void program_interface(struct OPTIONS *options)
{
    struct MENU menu;
    int endloop = 0;
    int row = 1;
    int aborted;
    int faborted;
    int filtered = 0;
    struct filterlist fl;
    struct othpoptions ofilter;
    char ifname[10];
    struct porttab *ports;

    draw_desktop();
    attrset(STDATTR);
    move(0, 1);
    printw("IPTraf 1.0.3");

    loadothfilter(&ofilter);
    loadaddports(&ports);

    about();

    initmenu(&menu, 11, 41, 7, 18);

    additem(&menu, " IP traffic ^m^onitor", "Displays current IP traffic information");
    additem(&menu, " General interface ^s^tatistics", "Displays some statistics for attached interfaces");
    additem(&menu, " ^D^etailed interface statistics", "Displays more statistics for a selected interface");
    additem(&menu, " TCP/UDP ser^v^ice monitor", "Displays statistics on TCP/UDP traffic");
    additem(&menu, " ^E^thernet station monitor", "Displays statistics on detected Ethernet stations");
    additem(&menu, " ^T^CP display filters", "Manages TCP display filters");
    additem(&menu, " Other ^p^rotocol filters", "Select which non-TCP protocols to display");
    additem(&menu, " ^O^ptions", "Set various program options");
    additem(&menu, " E^x^it", "Exits program");

    endloop = 0;

    do {
	showmenu(&menu);
	operatemenu(&menu, &row, &aborted);

	switch (row) {
	case 1:
	    ipmon(options->revlook, options->logging, filtered,
		  &fl, &ofilter, options->timeout);
	    break;
	case 2:
	    ifstats(options->logging, options->logspan);
	    break;
	case 3:
	    selectiface(ifname, &aborted);
	    if (!aborted)
		detstats(ifname, options->logging, options->logspan);
	    break;
	case 4:
	    selectiface(ifname, &aborted);
	    if (!aborted)
		servmon(ifname, ports, options->logging, options->logspan);
	    break;
	case 5:
	    hostmon(options->logging, options->logspan);
	    break;
	case 6:
	    tcpfilterselect(&fl, &filtered, &faborted);
	    break;
	case 7:
	    othfilterselect(&ofilter);
	    saveothfilter(&ofilter);
	    break;
	case 8:
	    setoptions(options, &ports);
	    saveoptions(options);
	    break;
	case 9:
	    endloop = 1;
	    break;
	}
    } while (!endloop);

    destroymenu(&menu);
    destroyporttab(ports);
    erase();
    update_panels();
    doupdate();
}

/*
 * Handler for the TERM signal.  There's nothing we can do for the KILL.
 */

void termhandler()
{
    erase();
    refresh();
    endwin();
    fprintf(stderr, "Received TERM signal\n");
    fprintf(stderr, "IPTraf terminated\n\n");
    exit(0);
}

/* 
 * Handler for the SIGSEGV, Segmentation Fault.  Tries to clear the screen
 * and issue a better message than "Segmentation fault".
 */

void segvhandler()
{
    erase();
    refresh();
    endwin();
    fprintf(stderr, "Fatal error: unable to allocate memory for a critical function\n");
    fprintf(stderr, "IPTraf terminated abnormally\n\n");
    exit(2);
}

void main(int argc, char **argv)
{
    struct OPTIONS options;

#ifndef ALLOWUSERS
    if (getuid() != 0) {
	fprintf(stderr, "\nIPTraf Version 1.0.3\n");
	fprintf(stderr, "Copyright (c) Gerard Paul Java 1997, 1998\n\n");
	fprintf(stderr, "This program can be run only by the system administrator\n\n");
	exit(1);
    }
#endif

    signal(SIGTERM, (void *) termhandler);
    signal(SIGSEGV, (void *) segvhandler);
    signal(SIGTSTP, SIG_IGN);
    signal(SIGINT, SIG_IGN);
    signal(SIGHUP, SIG_IGN);

    if (getenv("TERM") == NULL) {
	fprintf(stderr, "Your TERM variable is not set.\n");
	fprintf(stderr, "Please set it to an appropriate value.\n");
	exit(1);
    }
    loadoptions(&options);

    initscr();

    if ((LINES < 25) || (COLS < 80)) {
	endwin();
	fprintf(stderr, "\nThis program requires a screen size of at least 80 columns by 25 lines\n");
	fprintf(stderr, "Please resize your window\n\n");
	exit(1);
    }
    start_color();
    standardcolors(options.color);
    noecho();
    nonl();
    cbreak();
    curs_set(0);

    srpromisc(options.promisc);

    program_interface(&options);

    srpromisc(0);

    endwin();
}
